#!/bin/zsh -f

############################ Generating patches ############################

getversion () {
  . version
  echo $MAJOR_VERSION.$MINOR_VERSION.$PATCH_LEVEL.$INTERNAL_REVISION
}

compare_versions () {
  if cmp --silent ${TUNES}/version ${OLDTUNES}/version
  then
    echo 'Old and new distributions have same version number !'
    exit 1
  else
    return 0  # versions are different as expected.
  fi
}

mkpatch () {
  OLDVER=`cd $OLDTUNES ; getversion`
  PATCH=${PATCH_PREFIX}${OLDVER}-${TUNESVER}

  if [ -f $PATCH ] ; then ${=RM} $PATCH ; fi
  PATCH

  comprpatch

  #was:
  #export OLD=$OLDTUNES NEW=$TUNES
  #LOGDIR=	LOGFILE=tunesbin PATCHDIR
  #LOGDIR=src	LOGFILE=tunessrc PATCHDIR
  #LOGDIR=doc	LOGFILE=tunesdoc PATCHDIR
  #LOGDIR=arc	LOGFILE=tunesarc PATCHDIR
}

comprpatch () {
  #OLDVER=`cd $OLDTUNES ; getversion`
  #PATCH=${PATCH_PREFIX}${OLDVER}-${TUNESVER}
  ls -sl ${PATCH}
  ${=COMPR} ${PATCH}
  ls -sl ${PATCH}${=COMPRSUFFIX}
}

mkdiff () {
  diff --unified --new-file -P "$@"
  # to save a little space, use -C0 instead of --unified
}

function PATCH () {(
  # add as many -x as necessary to exclude all unneeded files
  # that may be generated or hand-written (temporary files, then)

  mkdiff --recursive -X doc/archive -x '*-list' \
    -x '.tmp*' -x 'patch.*' -x 'version.h' \
    -x '*.[oasilbxpZ]' \
    -x '*.obj' -x '*.bin' -x '*.exe' -x '*.co*' \
    -x '*.?rf' -x '*.lst' -x '*.map' \
    -x '*.tmp' -x '*.$$$' \
    -x '*~' -x '#*#' \
    -x '*.bak' -x '*.orig' -x '*.rej' \
    -x 'z[aeiou]p' -x foo -x bar -x test -x 'core*' \
    ${OLDTUNES} ${TUNES} >> ${PATCH}

  A=$? ; if [ $A = 1 ] ; then return 0 ; else return $A  ; fi
)}

cleanpatch () {
   BAD="`find . '(' -name '*.orig' \
		-o -name '*.rej' \
		-o -name '*.bak' ')' \
		-print`"
   echo $BAD >&2
   /bin/rm -f ${=BAD}
}



################### LOGGING FUNCTIONS (FOR INCREMENTAL SAVE) #############
########################### using zip/unzip ##############################
function LOGOUT () {(
  setopt NULL_GLOB
  if [ -n "$1" ] ; then LOGFILE=$1 ;
    if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
  LOGLOG=/tmp/Log-$HOST--$LOGFILE.log
  LOGSUBDIR=$3 ; if [ -z "$3" ] ; then echo -n > $LOGLOG ; fi
 
  cd $LOGDIR ; echo " Saving important files in $LOGDIR/$LOGSUBDIR:"

  FILES=$(eval echo $(cat ${LOGSUBDIR}.Logrc | grep -v '^[#lrA-Z]' |
	  ( while read c f x ; do echo ${LOGSUBDIR}$f ; done )))
  for i in ${=FILES} ; do
    if [ -d $i ]
    then
      I=$i/**/*
      for j in ${=I} 
      do if [ -d $j ] ; then echo $j/ ; else echo $j ; fi
      done
    else echo $i
    fi
  done  >> $LOGLOG
  zip -uy9r /$SVDIR/$LOGFILE ${=FILES}

  SUBDIRS=$(eval echo $(cat ${LOGSUBDIR}.Logrc | grep '^r' |
	  cut -d'	' -f1 | cut -d' ' -f2))
  for i in ${=SUBDIRS} ; do LOGOUT $LOGFILE $LOGDIR ${LOGSUBDIR}${i}/ ; done
)}

function LOGIN () {(
  if [ -n "$1" ] ; then LOGFILE=$1 ;
    if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
  cd $LOGDIR
  unzip -uo /$(syssvdir)/$LOGFILE
)}

function CLOG () {( # clean logfile
  setopt NULL_GLOB
  if [ -n "$1" ] ; then LOGFILE=$1 ;
    if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
  LOGLOG=/tmp/Log-$HOST--$LOGFILE.log

  GOODFILES=$(cat $LOGLOG)
  : GF $GOODFILES
  ZF=$(unzip -l /$SVDIR/$LOGFILE | cut -c28-)
  : ZF $ZF
  ZF=(${=ZF})
  : ZF $ZF
  ZF=$ZF[7,-3]
  : ZF $ZF
  ZIPPEDFILES=${=ZF}

  : ZIPPEDFILES=$ZIPPEDFILES
  : ZIPPY $ZIPPEDFILES[10,20]
  : GOODFILES=$GOODFILES
  : GOODY $GOODFILES[10,20]

  BADFILES=$(Set_Difference "$ZIPPEDFILES" "$GOODFILES")
  : BF $BADFILES
  echo zip -d /$SVDIR/$LOGFILE $BADFILES

  ### doesn't work: LOGOUT uses zip -r, so that all files below
  ### a zip -r 'ed directory will be considered as BAD !!!
)}

#function PATCHDIR () {(
#    setopt NULL_GLOB
#    if [ -n "$1" ] ; then LOGFILE=$1 ;
#      if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
#    LOGLOG=/tmp/Log-$HOST--$LOGFILE.log
#  
#    GOODFILES=$(cat $LOGLOG)
#    : GF $GOODFILES
#  
#    for i in ${=GOODFILES} ; do
#      #echo mkdiff $OLD/$LOGDIR/$i $NEW/$LOGDIR/$i
#      mkdiff $OLD/$LOGDIR/$i $NEW/$LOGDIR/$i
#    done
#  )>>${PATCH}
#}

function Is_Element () {(
  for i in ${=2} ; do if [ "$1" = "$i" ]; then return 0 ; fi ; done
  return 1
)}

function Set_Difference () {
# works for sets as IFS-separated list, not as arrays
  for i in ${=1} ; do if ! Is_Element $i $2 ; then echo $i ; fi ; done
}


########################## interactive use only #############################

function srcfiles () {
  echo info/{i386*,BIOS-API}.nfo src/boot/**/*.{ash,asm,inc} bin/install.sh
}

function t () {

}

#function main () {
  "$@"
#}
